-- |\    /| ____ ____  ____ _____   ____         _____
-- | \  / | |    |     |    |      |    | |\   | |
-- |  \/  | |___ ____  |___ |      |    | | \  | |____
-- |      | |        | |    |      |    | |  \ |     |
-- |      | |___ ____| |___ |____  |____| |   \| ____|
-- by Jeija and Minerd247
--
--
--
-- This mod adds mesecons[=minecraft redstone] and different receptors/effectors to minetest.
--
-- See the documentation on the forum for additional information, especially about crafting
--
--Quick Developer documentation for the mesecon API
--=================================================
--
--RECEPTORS
--
--A receptor is a node that emits power, e.g. a solar panel, a switch or a power plant.
--Usually you create two blocks per receptor that have to be switched when switching the on/off state: 
--	# An off-state node (e.g. jeija:mesecon_switch_off"
--	# An on-state node (e.g. jeija:mesecon_switch_on"
--The on-state node should be registered in the mesecon api, if the Mesecon circuit has to be
--recalculated. This can be done using
--
--mesecon:add_receptor_node(nodename)
--example: mesecon:add_receptor_node("jeija:mesecon_switch_on")
--
--Turning receptors on and off
--Usually the receptor has to turn on and off. For this, you have to
--	# Remove the node and replace it with the node in the other state (e.g. replace on by off)
--	# Send the event to the mesecon circuit by using the api functions
--		mesecon:receptor_on (pos) } These functions take the position of your receptor
--		mesecon:receptor_off(pos) } as their parameter.
--
--!! If a receptor node is removed, the circuit should be recalculated. This means you have to
--send an mesecon:receptor_off signal to the api when the function in minetest.register_on_dignode
--is called.
--
--EFFECTORS
--
--A receptor is a node that uses power and transfers the signal to a mechanical, optical whatever
--event. e.g. the meselamp, the movestone or the removestone.
--
--There are two callback functions for receptors.
--	# function mesecon:register_on_signal_on (action)
--	# function mesecon:register_on_signal_off(action)
--
--These functions will be called for each block next to a mesecon conductor.
--
--Example: The removestone
--The removestone only uses one callback: The mesecon:register_on_signal_on function
--
--mesecon:register_on_signal_on(function(pos, node) -- As the action prameter you have to use a function
--	if node.name=="jeija:removestone" then -- Check if it really is removestone. If you wouldn't use this, every node next to mesecons would be removed
--		minetest.env:remove_node(pos) -- The action: The removestone is removed
--	end -- end of if
--end) -- end of the function, )=end of the parameters of mesecon:register_on_signal_on



-- PUBLIC VARIABLES
mesecon={} -- contains all functions and all global variables
mesecon.actions_on={} -- Saves registered function callbacks for mesecon on
mesecon.actions_off={} -- Saves registered function callbacks for mesecon off
mesecon.pwr_srcs={} -- this is public for now
mesecon.wireless_receivers={}


-- MESECONS

minetest.register_node("jeija:mesecon_off", {
	drawtype = "raillike",
	tile_images = {"jeija_mesecon_off.png", "jeija_mesecon_curved_off.png", "jeija_mesecon_t_junction_off.png", "jeija_mesecon_crossing_off.png"},
	inventory_image = "jeija_mesecon_off.png",
	paramtype = "light",
	is_ground_content = true,
	walkable = false,
	selection_box = {
		type = "fixed",
	},
	material = minetest.digprop_constanttime(0.1),
})

minetest.register_node("jeija:mesecon_on", {
	drawtype = "raillike",
	tile_images = {"jeija_mesecon_on.png", "jeija_mesecon_curved_on.png", "jeija_mesecon_t_junction_on.png", "jeija_mesecon_crossing_on.png"},
	inventory_image = "jeija_mesecon_on.png",
	paramtype = "light",
	is_ground_content = true,
	walkable = false,
	selection_box = {
		type = "fixed",
	},
	material = minetest.digprop_constanttime(0.1),
	dug_item = 'node "jeija:mesecon_off" 1',
	light_source = LIGHT_MAX-11,
})

minetest.register_craft({
	output = 'node "jeija:mesecon_off" 16',
	recipe = {
		{'node "default:mese"'},
	}
})

function mesecon:is_power_on(p, x, y, z)
	local lpos = {}
	lpos.x=p.x+x
	lpos.y=p.y+y
	lpos.z=p.z+z
	local node = minetest.env:get_node(lpos)
	if node.name == "jeija:mesecon_on" or mesecon:is_receptor_node(node.name) then
		return 1
	end
	return 0
end

function mesecon:turnon(p, x, y, z, firstcall)
	local lpos = {}
	lpos.x=p.x+x
	lpos.y=p.y+y
	lpos.z=p.z+z

	mesecon:activate(lpos)

	local node = minetest.env:get_node(lpos)
	if node.name == "jeija:mesecon_off" then
		--minetest.env:remove_node(lpos)
		minetest.env:add_node(lpos, {name="jeija:mesecon_on"})
		nodeupdate(lpos)
	end
	if node.name == "jeija:mesecon_off" or firstcall then
		mesecon:turnon(lpos,  0,  0, -1, false)
		mesecon:turnon(lpos,  1,  0,  0, false)
		mesecon:turnon(lpos, -1,  0,  0, false)
		mesecon:turnon(lpos,  0,  0,  1, false)

		mesecon:turnon(lpos,  1,  1,  0, false)
		mesecon:turnon(lpos,  1, -1,  0, false)
		mesecon:turnon(lpos, -1,  1,  0, false)
		mesecon:turnon(lpos, -1, -1,  0, false)

		mesecon:turnon(lpos,  0,  1,  1, false)
		mesecon:turnon(lpos,  0, -1,  1, false)
		mesecon:turnon(lpos,  0,  1, -1, false)
		mesecon:turnon(lpos,  0, -1, -1, false)
	end
end

function mesecon:turnoff(pos, x, y, z, firstcall)
	local lpos = {}
	lpos.x=pos.x+x
	lpos.y=pos.y+y
	lpos.z=pos.z+z

	local node = minetest.env:get_node(lpos)
	local connected = 0
	local checked = {}

	if not mesecon:check_if_turnon(lpos) then
		mesecon:deactivate(lpos)
	end

	if not(firstcall) and connected==0 then
		connected=mesecon:connected_to_pw_src(lpos, 0, 0, 0, checked)	
	end

	if connected == 0 and  node.name == "jeija:mesecon_on" then
		--minetest.env:remove_node(lpos)
		minetest.env:add_node(lpos, {name="jeija:mesecon_off"})
		nodeupdate(lpos)
	end


	if node.name == "jeija:mesecon_on" or firstcall then
		if connected == 0 then
			mesecon:turnoff(lpos,  0,  0, -1, false)
			mesecon:turnoff(lpos,  1,  0,  0, false)
			mesecon:turnoff(lpos, -1,  0,  0, false)
			mesecon:turnoff(lpos,  0,  0,  1, false)
	
			mesecon:turnoff(lpos,  1,  1,  0, false)
			mesecon:turnoff(lpos,  1, -1,  0, false)
			mesecon:turnoff(lpos, -1,  1,  0, false)
			mesecon:turnoff(lpos, -1, -1,  0, false)
	
			mesecon:turnoff(lpos,  0,  1,  1, false)
			mesecon:turnoff(lpos,  0, -1,  1, false)
			mesecon:turnoff(lpos,  0,  1, -1, false)
			mesecon:turnoff(lpos,  0, -1, -1, false)
		end
	end
end


function mesecon:connected_to_pw_src(pos, x, y, z, checked, firstcall)
	local i=1
	local lpos = {}

	lpos.x=pos.x+x
	lpos.y=pos.y+y
	lpos.z=pos.z+z

	
	local node = minetest.env:get_node_or_nil(lpos)

	if not(node==nil) then
		repeat
			i=i+1
			if checked[i]==nil then checked[i]={} break end
			if  checked[i].x==lpos.x and checked[i].y==lpos.y and checked[i].z==lpos.z then 
				return 0
			end
		until false

		checked[i].x=lpos.x
		checked[i].y=lpos.y
		checked[i].z=lpos.z

		if mesecon:is_receptor_node(node.name) == true then -- receptor nodes (power sources) can be added using mesecon:add_receptor_node
			return 1
		end

		local pw_source_found=0
		if node.name=="jeija:mesecon_on" or firstcall then -- add other conductors here
			pw_source_found=pw_source_found+mesecon:connected_to_pw_src(lpos,  0,  0, -1, checked)
			pw_source_found=pw_source_found+mesecon:connected_to_pw_src(lpos,  1,  0,  0, checked)
			pw_source_found=pw_source_found+mesecon:connected_to_pw_src(lpos, -1,  0,  0, checked)
			pw_source_found=pw_source_found+mesecon:connected_to_pw_src(lpos,  0,  0,  1, checked)
	
			pw_source_found=pw_source_found+mesecon:connected_to_pw_src(lpos,  1,  1,  0, checked)
			pw_source_found=pw_source_found+mesecon:connected_to_pw_src(lpos,  1, -1,  0, checked)
			pw_source_found=pw_source_found+mesecon:connected_to_pw_src(lpos, -1,  1,  0, checked)
			pw_source_found=pw_source_found+mesecon:connected_to_pw_src(lpos, -1, -1,  0, checked)
	
			pw_source_found=pw_source_found+mesecon:connected_to_pw_src(lpos,  0,  1,  1, checked)
			pw_source_found=pw_source_found+mesecon:connected_to_pw_src(lpos,  0, -1,  1, checked)
			pw_source_found=pw_source_found+mesecon:connected_to_pw_src(lpos,  0,  1, -1, checked)
			pw_source_found=pw_source_found+mesecon:connected_to_pw_src(lpos,  0, -1, -1, checked)
			if pw_source_found > 0 then
				return 1
			end 
		end
	end
	return 0 
end

function mesecon:check_if_turnon(pos)
	local getactivated=0
	getactivated=getactivated+mesecon:is_power_on(pos,  0,  0, -1)
	getactivated=getactivated+mesecon:is_power_on(pos,  1,  0,  0)
	getactivated=getactivated+mesecon:is_power_on(pos, -1,  0,  0)
	getactivated=getactivated+mesecon:is_power_on(pos,  0,  0,  1)

	getactivated=getactivated+mesecon:is_power_on(pos,  1,  1,  0)
	getactivated=getactivated+mesecon:is_power_on(pos,  1, -1,  0)
	getactivated=getactivated+mesecon:is_power_on(pos, -1,  1,  0)
	getactivated=getactivated+mesecon:is_power_on(pos, -1, -1,  0)

	getactivated=getactivated+mesecon:is_power_on(pos,  0,  1,  1)
	getactivated=getactivated+mesecon:is_power_on(pos,  0, -1,  1)
	getactivated=getactivated+mesecon:is_power_on(pos,  0,  1, -1)
	getactivated=getactivated+mesecon:is_power_on(pos,  0, -1, -1)
	
	if getactivated > 0 then
		return true
	end
	return false
end

minetest.register_on_placenode(function(pos, newnode, placer)
	if mesecon:check_if_turnon(pos) then
		if newnode.name == "jeija:mesecon_off" then
			mesecon:turnon(pos, 0, 0, 0)		
		else
			mesecon:activate(pos)
		end
	end
end)

minetest.register_on_dignode(
	function(pos, oldnode, digger)
		if oldnode.name == "jeija:mesecon_on" then
			mesecon:turnoff(pos, 0, 0, 0, true)
		end	
	end
)



-- API API API API API API API API API API API API API API API API API API

function mesecon:add_receptor_node(nodename)
	local i=1
	repeat
		i=i+1
		if mesecon.pwr_srcs[i]==nil then break end
	until false
	mesecon.pwr_srcs[i]=nodename
end

function mesecon:receptor_on(pos)
	mesecon:turnon(pos, 0, 0, 0, true)
end

function mesecon:receptor_off(pos)
	mesecon:turnoff(pos, 0, 0, 0, true)
end

function mesecon:register_on_signal_on(action)
	local i	= 1	
	repeat
		i=i+1
		if mesecon.actions_on[i]==nil then break end
	until false
	mesecon.actions_on[i]=action
end

function mesecon:register_on_signal_off(action)
	local i	= 1	
	repeat
		i=i+1
		if mesecon.actions_off[i]==nil then break end
	until false
	mesecon.actions_off[i]=action
end



-- INTERNAL API


function mesecon:is_receptor_node(nodename)
	local i=1
	repeat
		i=i+1
		if mesecon.pwr_srcs[i]==nodename then return true end
	until mesecon.pwr_srcs[i]==nil
	return false
end

function mesecon:activate(pos)
	local node = minetest.env:get_node(pos)	
	local i = 1
	repeat
		i=i+1
		if mesecon.actions_on[i]~=nil then mesecon.actions_on[i](pos, node) 
		else break			
		end
	until false
end

function mesecon:deactivate(pos)
	local node = minetest.env:get_node(pos)	
	local i = 1
	local checked={}		
	repeat
		i=i+1
		if mesecon.actions_off[i]~=nil then mesecon.actions_off[i](pos, node) 
		else break			
		end
	until false
end


mesecon:register_on_signal_on(function(pos, node)
	if node.name=="jeija:meselamp_off" then
		--minetest.env:remove_node(pos)
		minetest.env:add_node(pos, {name="jeija:meselamp_on"})
		nodeupdate(pos)
	end
end)

mesecon:register_on_signal_off(function(pos, node)
	if node.name=="jeija:meselamp_on" then
		--minetest.env:remove_node(pos)
		minetest.env:add_node(pos, {name="jeija:meselamp_off"})
		nodeupdate(pos)
	end
end)









-- The POWER_PLANT

minetest.register_node("jeija:power_plant", {
	drawtype = "plantlike",
	visual_scale = 1,
	tile_images = {"jeija_power_plant.png"},
	inventory_image = "jeija_power_plant.png",
	paramtype = "light",
	walkable = false,
	material = minetest.digprop_leaveslike(0.2),
	light_source = LIGHT_MAX-9,
})

minetest.register_craft({
	output = 'node "jeija:power_plant" 1',
	recipe = {
		{'node "jeija:mesecon_off"'},
		{'node "jeija:mesecon_off"'},
		{'node "default:junglegrass"'},
	}
})

minetest.register_on_placenode(function(pos, newnode, placer)
	if newnode.name == "jeija:power_plant" then
		mesecon:receptor_on(pos)
	end
end)

minetest.register_on_dignode(
	function(pos, oldnode, digger)
		if oldnode.name == "jeija:power_plant" then
			mesecon:receptor_off(pos)
		end	
	end
)

mesecon:add_receptor_node("jeija:power_plant")


-- The BLINKY_PLANT

minetest.register_node("jeija:blinky_plant_off", {
	drawtype = "plantlike",
	visual_scale = 1,
	tile_images = {"jeija_blinky_plant_off.png"},
	inventory_image = "jeija_blinky_plant_off.png",
	paramtype = "light",
	walkable = false,
	material = minetest.digprop_leaveslike(0.2),
})

minetest.register_node("jeija:blinky_plant_on", {
	drawtype = "plantlike",
	visual_scale = 1,
	tile_images = {"jeija_blinky_plant_on.png"},
	inventory_image = "jeija_blinky_plant_off.png",
	paramtype = "light",
	walkable = false,
	material = minetest.digprop_leaveslike(0.2),
	dug_item='node "jeija:blinky_plant_off" 1',
	light_source = LIGHT_MAX-7,
})

minetest.register_craft({
	output = 'node "jeija:blinky_plant_off" 1',
	recipe = {
	{'','node "jeija:mesecon_off"',''},
	{'','node "jeija:mesecon_off"',''},
	{'node "default:junglegrass"','node "default:junglegrass"','node "default:junglegrass"'},
	}
})

minetest.register_abm(
	{nodenames = {"jeija:blinky_plant_off"},
	interval = 3.0,
	chance = 1,
	action = function(pos, node, active_object_count, active_object_count_wider)
		--minetest.env:remove_node(pos)
		minetest.env:add_node(pos, {name="jeija:blinky_plant_on"})
		nodeupdate(pos)	
		mesecon:receptor_on(pos)
	end,
})

minetest.register_abm({
	nodenames = {"jeija:blinky_plant_on"},
	interval = 3.0,
	chance = 1,
	action = function(pos, node, active_object_count, active_object_count_wider)
		--minetest.env:remove_node(pos)
		minetest.env:add_node(pos, {name="jeija:blinky_plant_off"})
		nodeupdate(pos)	
		mesecon:receptor_off(pos)
	end,
})

mesecon:add_receptor_node("jeija:blinky_plant_on")

minetest.register_on_dignode(
	function(pos, oldnode, digger)
		if oldnode.name == "jeija:blinky_plant_on" then
			mesecon:receptor_off(pos)
		end
	end
)


-- Solar Panel

minetest.register_craftitem("jeija:silicon", {
	image = "jeija_silicon.png",
	on_place_on_ground = minetest.craftitem_place_item,
})


minetest.register_node("jeija:solar_panel", {
	drawtype = "raillike",
	tile_images = {"jeija_solar_panel.png"},
	inventory_image = "jeija_solar_panel.png",
	paramtype = "light",
	walkable = false,
	is_ground_content = true,
	selection_box = {
		type = "fixed",
	},
	furnace_burntime = 5,
	material = minetest.digprop_dirtlike(0.1),
})

minetest.register_craft({
	output = 'craft "jeija:silicon" 4',
	recipe = {
		{'node "default:sand"', 'node "default:sand"'},
		{'node "default:sand"', 'craft "default:steel_ingot"'},
	}
})

minetest.register_craft({
	output = 'node "jeija:solar_panel" 1',
	recipe = {
		{'craft "jeija:silicon"', 'craft "jeija:silicon"'},
		{'craft "jeija:silicon"', 'craft "jeija:silicon"'},
	}
})

minetest.register_abm(
	{nodenames = {"jeija:solar_panel"},
	interval = 0.1,
	chance = 1,
	action = function(pos, node, active_object_count, active_object_count_wider)
		local light = minetest.env:get_node_light(pos, nil)
		if light == nil then light = 0 end
		if light >= 13 then
			mesecon:receptor_on(pos)
		else
			mesecon:receptor_off(pos)
		end
	end,
})


-- MESELAMPS
minetest.register_node("jeija:meselamp_on", {
	drawtype = "torchlike",
	tile_images = {"jeija_meselamp_on_floor_on.png", "jeija_meselamp_on_ceiling_on.png", "jeija_meselamp_on.png"},
	inventory_image = "jeija_meselamp_on_floor_on.png",
	paramtype = "light",
	sunlight_propagates = true,
	walkable = false,
	wall_mounted = false,
	light_source = LIGHT_MAX,
	selection_box = {
		type = "fixed",
		fixed = {-0.38, -0.5, -0.1, 0.38, -0.2, 0.1},
	},
	material = minetest.digprop_constanttime(0.1),
	dug_item='node "jeija:meselamp_off" 1',
})

minetest.register_node("jeija:meselamp_off", {
	drawtype = "torchlike",
	tile_images = {"jeija_meselamp_on_floor_off.png", "jeija_meselamp_on_ceiling_off.png", "jeija_meselamp_off.png"},
	inventory_image = "jeija_meselamp_on_floor_off.png",
	paramtype = "light",
	sunlight_propagates = true,
	walkable = false,
	wall_mounted = false,
	selection_box = {
		type = "fixed",
		fixed = {-0.38, -0.5, -0.1, 0.38, -0.2, 0.1},
	},
	material = minetest.digprop_constanttime(0.1),
})

minetest.register_craft({
	output = 'node "jeija:meselamp_off" 1',
	recipe = {
		{'', 'node "default:glass"', ''},
		{'node "jeija:mesecon_off"', 'craft "default:steel_ingot"', 'node "jeija:mesecon_off"'},
		{'', 'node "default:glass"', ''},
	}
})

-- MOVESTONE

function jeija_get_movestone_direction(pos)
	getactivated=0
	local direction = {x=0, y=0, z=0}
	local lpos=pos
	
	lpos.x=pos.x+0.499

	getactivated=getactivated+mesecon:is_power_on(lpos,  0,  0, -1)
	getactivated=getactivated+mesecon:is_power_on(lpos,  0, -1, -1)
	getactivated=getactivated+mesecon:is_power_on(lpos,  0,  1, -1)

	if getactivated>0 then direction.x=-1 return direction end
	lpos=pos
	lpos.x=pos.x-0.499

	getactivated=getactivated+mesecon:is_power_on(lpos,  0,  1,  1)
	getactivated=getactivated+mesecon:is_power_on(lpos,  0, -1,  1)
	getactivated=getactivated+mesecon:is_power_on(lpos,  0,  0,  1)

	if getactivated>0 then direction.x=1 return direction end
	lpos=pos
	lpos.z=pos.z+0.499

	getactivated=getactivated+mesecon:is_power_on(lpos,  1,  0,  0)
	getactivated=getactivated+mesecon:is_power_on(lpos,  1,  1,  0)
	getactivated=getactivated+mesecon:is_power_on(lpos,  1, -1,  0)

	if getactivated>0 then direction.z=-1 return direction end
	lpos=pos
	lpos.z=pos.z-0.499

	getactivated=getactivated+mesecon:is_power_on(lpos, -1,  1,  0)
	getactivated=getactivated+mesecon:is_power_on(lpos, -1, -1,  0)
	getactivated=getactivated+mesecon:is_power_on(lpos, -1,  0,  0)
	if getactivated>0 then direction.z=1 return direction end
	return direction
end

minetest.register_node("jeija:movestone", {
	tile_images = {"jeija_movestone_side.png", "jeija_movestone_side.png", "jeija_movestone_side.png", "jeija_movestone_side.png", "jeija_movestone_arrows.png", "jeija_movestone_arrows.png"},
	inventory_image = minetest.inventorycube("jeija_movestone_arrows.png", "jeija_movestone_side.png", "jeija_movestone_side.png"),
	paramtype = "facedir_simple",
	material = minetest.digprop_stonelike(0.8),
})

minetest.register_entity("jeija:movestone_entity", {
	physical = false,
	visual = "sprite",
	textures = {"jeija_movestone_side.png", "jeija_movestone_side.png", "jeija_movestone_side.png", "jeija_movestone_side.png", "jeija_movestone_arrows.png", "jeija_movestone_arrows.png"},
	collisionbox = {-0.5,-0.5,-0.5, 0.5,0.5,0.5},
	visual = "cube",
	--on_activate = function(self, staticdata)
		--self.object:setsprite({x=0,y=0}, 1, 0, true)
		--self.object:setvelocity({x=-3, y=0, z=0})
	--end,

	on_punch = function(self, hitter)
		self.object:remove()
		hitter:add_to_inventory('node jeija:movestone 1')
	end,

	on_step = function(self, dtime)
		local pos = self.object:getpos()
		local colp = pos
		local velocity={}
		local direction=jeija_get_movestone_direction(colp)


		if (direction.x==0 and direction.y==0 and direction.z==0)
		or (minetest.env:get_node_or_nil(pos).name ~="air" 
		and minetest.env:get_node_or_nil(pos).name ~= nil) then
			minetest.env:add_node(pos, {name="jeija:movestone"})
			self.object:remove()
			return		
		end 

		velocity.x=direction.x*3
		velocity.y=direction.y*3
		velocity.z=direction.z*3

		self.object:setvelocity(velocity)

		local np = {x=pos.x+direction.x, y=pos.y+direction.y, z=pos.z+direction.z}	
		local coln = minetest.env:get_node(np)
		if coln.name ~= "air" and coln.name ~="water" then
			local thisp= {x=pos.x, y=pos.y, z=pos.z}
			local thisnode=minetest.env:get_node(thisp)
			local nextnode={}
			minetest.env:remove_node(thisp)
			repeat
				thisp.x=thisp.x+direction.x
				thisp.y=thisp.y+direction.y
				thisp.z=thisp.z+direction.z
				nextnode=minetest.env:get_node(thisp)
				minetest.env:add_node(thisp, {name=thisnode.name})
				nodeupdate(thisp)
				thisnode=nextnode
			until thisnode.name=="air" or thisnode.name=="ignore" or thisnode.name=="default:water" or thisnode.name=="default:water_flowing"
		end
	end
})

minetest.register_craft({
	output = 'node "jeija:movestone" 2',
	recipe = {
		{'node "default:stone"', 'node "default:stone"', 'node "default:stone"'},
		{'node "jeija:mesecon_off"', 'node "jeija:mesecon_off"', 'node "jeija:mesecon_off"'},
		{'node "default:stone"', 'node "default:stone"', 'node "default:stone"'},
	}
})


mesecon:register_on_signal_on(function (pos, node)
	if node.name=="jeija:movestone" then
		minetest.env:remove_node(pos)
		nodeupdate(pos)
		minetest.env:add_entity(pos, "jeija:movestone_entity")
	end
end)

--GLUE
minetest.register_craftitem("jeija:glue", {
	image = "jeija_glue.png",
	on_place_on_ground = minetest.craftitem_place_item,
})

minetest.register_craft({
	output = 'craft "jeija:glue" 2',
	recipe = {
		{'node "default:junglegrass"', 'node "default:junglegrass"'},
		{'node "default:junglegrass"', 'node "default:junglegrass"'},
	}
})

-- STICKY_MOVESTONE

minetest.register_node("jeija:sticky_movestone", {
	tile_images = {"jeija_movestone_side.png", "jeija_movestone_side.png", "jeija_movestone_side.png", "jeija_movestone_side.png", "jeija_sticky_movestone.png", "jeija_sticky_movestone.png"},
	inventory_image = minetest.inventorycube("jeija_sticky_movestone.png", "jeija_movestone_side.png", "jeija_movestone_side.png"),
	paramtype = "facedir_simple",
	material = minetest.digprop_stonelike(0.8),
})

minetest.register_craft({
	output = 'node "jeija:sticky_movestone" 2',
	recipe = {
		{'craft "jeija:glue"', 'node "jeija:movestone"', 'craft "jeija:glue"'},
	}
})

minetest.register_entity("jeija:sticky_movestone_entity", {
	physical = false,
	visual = "sprite",
	textures = {"jeija_movestone_side.png", "jeija_movestone_side.png", "jeija_movestone_side.png", "jeija_movestone_side.png", "jeija_sticky_movestone.png", "jeija_sticky_movestone.png"},
	collisionbox = {-0.5,-0.5,-0.5, 0.5,0.5,0.5},
	visual = "cube",

	on_punch = function(self, hitter)
		self.object:remove()
		hitter:add_to_inventory('node jeija:sticky_movestone 1')
	end,

	on_step = function(self, dtime)
		local pos = self.object:getpos()
		local colp = pos
		local velocity={}
		local direction=jeija_get_movestone_direction(colp)

		velocity.x=direction.x*3
		velocity.y=direction.y*3
		velocity.z=direction.z*3

		self.object:setvelocity(velocity)

		local np = {x=pos.x+direction.x, y=pos.y+direction.y, z=pos.z+direction.z}	
		local coln = minetest.env:get_node(np)
		if coln.name ~= "air" and coln.name ~="water" then
			local thisp= {x=pos.x, y=pos.y, z=pos.z}
			local thisnode=minetest.env:get_node(thisp)
			local nextnode={}
			minetest.env:remove_node(thisp)
			repeat
				thisp.x=thisp.x+direction.x
				thisp.y=thisp.y+direction.y
				thisp.z=thisp.z+direction.z
				nextnode=minetest.env:get_node(thisp)
				minetest.env:add_node(thisp, {name=thisnode.name})
				nodeupdate(thisp)
				thisnode=nextnode
			until thisnode.name=="air" or thisnode.name=="ignore" or thisnode.name=="default:water" or thisnode.name=="default:water_flowing"
		end

		

		--STICKY:
		local np1 = {x=pos.x-direction.x*0.9, y=pos.y-direction.y*0.9, z=pos.z-direction.z*0.9} -- 1 away
		local coln1 = minetest.env:get_node(np1)
		local np2 = {x=pos.x-direction.x*1.9, y=pos.y-direction.y*1.9, z=pos.z-direction.z*1.9} -- 2 away
		local coln2 = minetest.env:get_node(np2)


		if (coln1.name == "air" or coln1.name =="water") and (coln2.name~="air" and coln2.name ~= water) then
			thisp= np2
			local newpos={}
			local oldpos={}
			repeat
				newpos.x=thisp.x+direction.x
				newpos.y=thisp.y+direction.y
				newpos.z=thisp.z+direction.z
				minetest.env:add_node(newpos, {name=minetest.env:get_node(thisp).name})
				nodeupdate(newpos)
				oldpos={x=thisp.x, y=thisp.y, z=thisp.z}
				thisp.x=thisp.x-direction.x
				thisp.y=thisp.y-direction.y
				thisp.z=thisp.z-direction.z
			until minetest.env:get_node(thisp).name=="air" or minetest.env:get_node(thisp).name=="ignore" or minetest.env:get_node(thisp).name=="default:water" or minetest.env:get_node(thisp).name=="default:water_flowing"
			minetest.env:remove_node(oldpos)
		end


		local checkpos={x=pos.x+direction.x*0.2, y=pos.y+direction.x*0.2, z=pos.z+direction.z*0.2}
		if (direction.x==0 and direction.y==0 and direction.z==0)
		or (minetest.env:get_node_or_nil(checkpos).name ~="air" 
		and minetest.env:get_node_or_nil(checkpos).name ~= nil) then
			minetest.env:add_node(pos, {name="jeija:sticky_movestone"})
			self.object:remove()
			return		
		end 
	end
})

minetest.register_craft({
	output = 'node "jeija:sticky_movestone" 2',
	recipe = {
		{'node "default:stone"', 'node "default:stone"', 'node "default:stone"'},
		{'node "jeija:mesecon_off"', 'node "jeija:mesecon_off"', 'node "default:tree"'},
		{'node "default:stone"', 'node "default:stone"', 'node "default:stone"'},
	}
})


mesecon:register_on_signal_on(function (pos, node)
	if node.name=="jeija:sticky_movestone" then
		minetest.env:remove_node(pos)
		nodeupdate(pos)
		minetest.env:add_entity(pos, "jeija:sticky_movestone_entity")
	end
end)

-- HYDRO_TURBINE

minetest.register_node("jeija:hydro_turbine_off", {
	tile_images = {"jeija_hydro_turbine_off.png", "jeija_hydro_turbine_off.png", "jeija_hydro_turbine_off.png", "jeija_hydro_turbine_off.png", "jeija_hydro_turbine_off.png", "jeija_hydro_turbine_off.png"},
	inventory_image = minetest.inventorycube("jeija_hydro_turbine_off.png", "jeija_hydro_turbine_off.png", "jeija_hydro_turbine_off.png"),
	material = minetest.digprop_constanttime(0.5),
})

minetest.register_node("jeija:hydro_turbine_on", {
	tile_images = {"jeija_hydro_turbine_on.png", "jeija_hydro_turbine_on.png", "jeija_hydro_turbine_on.png", "jeija_hydro_turbine_on.png", "jeija_hydro_turbine_on.png", "jeija_hydro_turbine_on.png"},
	inventory_image = minetest.inventorycube("jeija_hydro_turbine_on.png", "jeija_hydro_turbine_on.png", "jeija_hydro_turbine_on.png"),
	dug_item = 'node "jeija:hydro_turbine_off" 1',
	material = minetest.digprop_constanttime(0.5),
})



minetest.register_abm({
nodenames = {"jeija:hydro_turbine_off"},
	interval = 1,
	chance = 1,
	action = function(pos, node, active_object_count, active_object_count_wider)
		local waterpos={x=pos.x, y=pos.y+1, z=pos.z}
		if minetest.env:get_node(waterpos).name=="default:water_flowing" then
			--minetest.env:remove_node(pos)
			minetest.env:add_node(pos, {name="jeija:hydro_turbine_on"})
			nodeupdate(pos)
			mesecon:receptor_on(pos)
		end
	end,
})

minetest.register_abm({
nodenames = {"jeija:hydro_turbine_on"},
	interval = 1,
	chance = 1,
	action = function(pos, node, active_object_count, active_object_count_wider)
		local waterpos={x=pos.x, y=pos.y+1, z=pos.z}
		if minetest.env:get_node(waterpos).name~="default:water_flowing" then
			--minetest.env:remove_node(pos)
			minetest.env:add_node(pos, {name="jeija:hydro_turbine_off"})
			nodeupdate(pos)
			mesecon:receptor_off(pos)
		end
	end,
})

mesecon:add_receptor_node("jeija:hydro_turbine_on")

minetest.register_craft({
	output = 'node "jeija:hydro_turbine_off" 2',
	recipe = {
	{'','craft "default:stick"', ''},
	{'craft "default:stick"', 'craft "default:steel_ingot"', 'craft "default:stick"'},
	{'','craft "default:stick"', ''},
	}
})


-- MESECON_SWITCH

minetest.register_node("jeija:mesecon_switch_off", {
	tile_images = {"jeija_mesecon_switch_side.png", "jeija_mesecon_switch_side.png", "jeija_mesecon_switch_side.png", "jeija_mesecon_switch_side.png", "jeija_mesecon_switch_side.png", "jeija_mesecon_switch_off.png"},
	inventory_image = minetest.inventorycube("jeija_mesecon_switch_side.png", "jeija_mesecon_switch_side.png", "jeija_mesecon_switch_off.png"),
	paramtype = "facedir_simple",
	material = minetest.digprop_constanttime(0.5),
})

minetest.register_node("jeija:mesecon_switch_on", {
	tile_images = {"jeija_mesecon_switch_side.png", "jeija_mesecon_switch_side.png", "jeija_mesecon_switch_side.png", "jeija_mesecon_switch_side.png", "jeija_mesecon_switch_side.png", "jeija_mesecon_switch_on.png"},
	inventory_image = minetest.inventorycube("jeija_mesecon_switch_side.png", "jeija_mesecon_switch_side.png", "jeija_mesecon_switch_on.png"),
	paramtype = "facedir_simple",
	material = minetest.digprop_constanttime(0.5),
	dug_item='node "jeija:mesecon_switch_off" 1',
})

mesecon:add_receptor_node("jeija:mesecon_switch_on")

minetest.register_on_punchnode(function(pos, node, puncher)
	if node.name == "jeija:mesecon_switch_on" then
		--minetest.env:remove_node(pos)
		minetest.env:add_node(pos, {name="jeija:mesecon_switch_off"})
		nodeupdate(pos)
		mesecon:receptor_off(pos)
	end
	if node.name == "jeija:mesecon_switch_off" then
		--minetest.env:remove_node(pos)
		minetest.env:add_node(pos, {name="jeija:mesecon_switch_on"})
		nodeupdate(pos)
		mesecon:receptor_on(pos)
	end
end)

minetest.register_on_dignode(
	function(pos, oldnode, digger)
		if oldnode.name == "jeija:mesecon_switch_on" then
			mesecon:receptor_off(pos)
		end
	end
)

minetest.register_craft({
	output = 'node "jeija:mesecon_switch_off" 2',
	recipe = {
		{'craft "default:steel_ingot"', 'node "default:cobble"', 'craft "default:steel_ingot"'},
		{'node "jeija:mesecon_off"','', 'node "jeija:mesecon_off"'},
	}
})

--Launch TNT

mesecon:register_on_signal_on(function(pos, node)
	if node.name=="experimental:tnt" then
		minetest.env:remove_node(pos)
		minetest.env:add_entity(pos, "experimental:tnt")
		nodeupdate(pos)
	end
end)

-- REMOVE_STONE

minetest.register_node("jeija:removestone", {
	tile_images = {"jeija_removestone.png"},
	inventory_image = minetest.inventorycube("jeija_removestone_inv.png"),
	material = minetest.digprop_stonelike(1.0),
})

minetest.register_craft({
	output = 'node "jeija:removestone" 4',
	recipe = {
		{'', 'node "default:cobble"',''},
		{'node "default:cobble"', 'node "jeija:mesecon_off"', 'node "default:cobble"'},
		{'', 'node "default:cobble"',''},
	}
})

mesecon:register_on_signal_on(function(pos, node)
	if node.name=="jeija:removestone" then
		minetest.env:remove_node(pos)
	end
end)

-- IC
minetest.register_craftitem("jeija:ic", {
	image = "jeija_ic.png",
	on_place_on_ground = minetest.craftitem_place_item,
})

minetest.register_craft({
	output = 'craft "jeija:ic" 2',
	recipe = {
		{'craft "jeija:silicon"', 'craft "jeija:silicon"', 'node "jeija:mesecon_off"'},
		{'craft "jeija:silicon"', 'craft "jeija:silicon"', 'node "jeija:mesecon_off"'},
		{'node "jeija:mesecon_off"', 'node "jeija:mesecon_off"', ''},
	}
})

--COMMON WIRELESS FUNCTIONS

function mesecon:read_wlre_from_file()
	print "Reading Mesecon Data..."
	mesecon_file=io.open("mesecon_data", "r")
	local row
	local i=1
	if mesecon_file==nil then return end
	row=mesecon_file:read()
	while row~=nil do
		mesecon.wireless_receivers[i]={}
		mesecon.wireless_receivers[i].pos={}
		mesecon.wireless_receivers[i].pos.x=tonumber(mesecon_file:read())
		mesecon.wireless_receivers[i].pos.y=tonumber(mesecon_file:read())
		mesecon.wireless_receivers[i].pos.z=tonumber(mesecon_file:read())
		mesecon.wireless_receivers[i].channel=mesecon_file:read()
		mesecon.wireless_receivers[i].requested_state=tonumber(mesecon_file:read())
		mesecon.wireless_receivers[i].inverting=tonumber(mesecon_file:read())
		i=i+1
		row=mesecon_file:read()
	end
	mesecon_file:close()	
	print "Finished Reading Mesecon Data..."
end


function mesecon:register_wireless_receiver(pos, inverting)
	local i	= 1	
	repeat
		if mesecon.wireless_receivers[i]==nil then break end
		i=i+1
	until false


	local node_under_pos={}
	node_under_pos.x=pos.x
	node_under_pos.y=pos.y
	node_under_pos.z=pos.z

	node_under_pos.y=node_under_pos.y-1
	local node_under=minetest.env:get_node(node_under_pos)
	mesecon.wireless_receivers[i]={}
	mesecon.wireless_receivers[i].pos={}
	mesecon.wireless_receivers[i].pos.x=pos.x
	mesecon.wireless_receivers[i].pos.y=pos.y
	mesecon.wireless_receivers[i].pos.z=pos.z
	mesecon.wireless_receivers[i].channel=node_under.name
	mesecon.wireless_receivers[i].requested_state=0
	mesecon.wireless_receivers[i].inverting=inverting
end

function mesecon:remove_wireless_receiver(pos)
	local i = 1
	while mesecon.wireless_receivers[i]~=nil do
		if mesecon.wireless_receivers[i].pos.x==pos.x and
		   mesecon.wireless_receivers[i].pos.y==pos.y and
		   mesecon.wireless_receivers[i].pos.z==pos.z then
			mesecon.wireless_receivers[i]=nil
			break
		end
		i=i+1
	end
end

function mesecon:set_wlre_channel(pos, channel)
	--local i = 1
	--while mesecon.wireless_receivers[i]~=nil do
	--	if tonumber(mesecon.wireless_receivers[i].pos.x)==tonumber(pos.x) and
	--	   tonumber(mesecon.wireless_receivers[i].pos.y)==tonumber(pos.y) and
	--	   tonumber(mesecon.wireless_receivers[i].pos.z)==tonumber(pos.z) then
	--		mesecon.wireless_receivers[i].channel=channel
	--		break
	--	end
	--	i=i+1
	--end
	local wlre=mesecon:get_wlre(pos)
	if wlre~=nil then 
		wlre.channel=channel
	end
end

function mesecon:get_wlre(pos)
	local i=1
	while mesecon.wireless_receivers[i]~=nil do
		if mesecon.wireless_receivers[i].pos.x==pos.x and
		   mesecon.wireless_receivers[i].pos.y==pos.y and
		   mesecon.wireless_receivers[i].pos.z==pos.z then
			return mesecon.wireless_receivers[i]
		end
		i=i+1
	end
end

minetest.register_on_placenode(function(pos, newnode, placer)
	pos.y=pos.y+1
	if minetest.env:get_node(pos).name == "jeija:wireless_receiver_off" or
	   minetest.env:get_node(pos).name == "jeija:wireless_receiver_on"  or
	   minetest.env:get_node(pos).name == "jeija:wireless_inverter_off" or
	   minetest.env:get_node(pos).name == "jeija:wireless_inverter_on" then
		mesecon:set_wlre_channel(pos, newnode.name)
	end
end)

minetest.register_on_dignode(
	function(pos, oldnode, digger)
		local channel
		pos.y=pos.y+1
		if minetest.env:get_node(pos).name == "jeija:wireless_receiver_on" or
		   minetest.env:get_node(pos).name == "jeija:wireless_receiver_off" or
		   minetest.env:get_node(pos).name == "jeija:wireless_inverter_on" or
		   minetest.env:get_node(pos).name == "jeija:wireless_inverter_off" then
			mesecon:set_wlre_channel(pos, "air")
		end	
	end
)

minetest.register_abm(
	{nodenames = {"jeija:wireless_receiver_on", "jeija:wireless_receiver_off",
		      "jeija:wireless_inverter_on", "jeija:wireless_inverter_off"},
	interval = 1.0,
	chance = 1,
	action = function(pos, node, active_object_count, active_object_count_wider)
		local wlre=mesecon:get_wlre(pos)
		if (wlre==nil) then return end
		
		if node.name=="jeija:wireless_receiver_on" and wlre.requested_state==0  then
			minetest.env:add_node(pos, {name="jeija:wireless_receiver_off"})
			mesecon:receptor_off(pos)
		end
		if node.name=="jeija:wireless_receiver_off" and wlre.requested_state==1  then
			minetest.env:add_node(pos, {name="jeija:wireless_receiver_on"})
			mesecon:receptor_on(pos)
		end
		if node.name=="jeija:wireless_inverter_off" and wlre.requested_state==0 and wlre.inverting==1 then
			minetest.env:add_node(pos, {name="jeija:wireless_inverter_on"})
			mesecon:receptor_on(pos)
		end
		if node.name=="jeija:wireless_inverter_on" and wlre.requested_state==1 and wlre.inverting==1 then
			minetest.env:add_node(pos, {name="jeija:wireless_inverter_off"})
			mesecon:receptor_off(pos)
		end
	end,
})

--WIRELESS RECEIVER

minetest.register_node("jeija:wireless_receiver_off", {
	tile_images = {"jeija_wireless_receiver_tb_off.png", "jeija_wireless_receiver_tb_off.png", "jeija_wireless_receiver_off.png", "jeija_wireless_receiver_off.png", "jeija_wireless_receiver_off.png", "jeija_wireless_receiver_off.png"},
	inventory_image = minetest.inventorycube("jeija_wireless_receiver_off.png"),
	material = minetest.digprop_constanttime(0.8),
})

minetest.register_node("jeija:wireless_receiver_on", {
	tile_images = {"jeija_wireless_receiver_tb_on.png", "jeija_wireless_receiver_tb_on.png", "jeija_wireless_receiver_on.png", "jeija_wireless_receiver_on.png", "jeija_wireless_receiver_on.png", "jeija_wireless_receiver_on.png"},
	inventory_image = minetest.inventorycube("jeija_wireless_receiver_on.png"),
	material = minetest.digprop_constanttime(0.8),
	dug_item = 'node "jeija:wireless_receiver_off" 1'
})

minetest.register_craft({
	output = 'node "jeija:wireless_receiver_off" 2',
	recipe = {
		{'', 'node "jeija:mesecon_off"', ''},
		{'', 'node "jeija:mesecon_off"', ''},
		{'', 'craft "jeija:ic"', ''},
	}
})

minetest.register_on_placenode(function(pos, newnode, placer)
	if newnode.name == "jeija:wireless_receiver_off" then
		mesecon:register_wireless_receiver(pos, 0)
	end
end)

minetest.register_on_dignode(
	function(pos, oldnode, digger)
		if oldnode.name == "jeija:wireless_receiver_on" then
			mesecon:remove_wireless_receiver(pos)
			mesecon:receptor_off(pos)
		end	
		if oldnode.name == "jeija:wireless_receiver_off" then
			mesecon:remove_wireless_receiver(pos)		
		end
	end
)

minetest.register_abm( -- SAVE WIRELESS RECEIVERS TO FILE
	{nodenames = {"jeija:wireless_receiver_off", "jeija:wireless_receiver_on", "jeija:wireless_inverter_on", "jeija:wireless_inverter_off"},
	interval = 10,
	chance = 1,
	action = function(pos, node, active_object_count, active_object_count_wider)
		local mesecon_file = io.open("mesecon_data", "w")
		local i=1
		while mesecon.wireless_receivers[i]~=nil do
			mesecon_file:write("NEXT\n")
			mesecon_file:write(mesecon.wireless_receivers[i].pos.x.."\n")
			mesecon_file:write(mesecon.wireless_receivers[i].pos.y.."\n")
			mesecon_file:write(mesecon.wireless_receivers[i].pos.z.."\n")
			mesecon_file:write(mesecon.wireless_receivers[i].channel.."\n")
			mesecon_file:write(mesecon.wireless_receivers[i].requested_state.."\n")
			mesecon_file:write(mesecon.wireless_receivers[i].inverting.."\n")
			i=i+1
		end
		mesecon_file:close()
	end, 
})

mesecon:add_receptor_node("jeija:wireless_receiver_on")

-- WIRELESS INVERTER OFF/ON BELONGS TO THE OUTPUT STATE (ON=INPUT OFF)

minetest.register_node("jeija:wireless_inverter_off", {
	tile_images = {"jeija_wireless_inverter_tb.png", "jeija_wireless_inverter_tb.png", "jeija_wireless_inverter_off.png", "jeija_wireless_inverter_off.png", "jeija_wireless_inverter_off.png", "jeija_wireless_inverter_off.png"},
	inventory_image = minetest.inventorycube("jeija_wireless_inverter_off.png"),
	material = minetest.digprop_constanttime(0.8),
	dug_item = 'node "jeija:wireless_inverter_on" 1'
})

minetest.register_node("jeija:wireless_inverter_on", {
	tile_images = {"jeija_wireless_inverter_tb.png", "jeija_wireless_inverter_tb.png", "jeija_wireless_inverter_on.png", "jeija_wireless_inverter_on.png", "jeija_wireless_inverter_on.png", "jeija_wireless_inverter_on.png"},
	inventory_image = minetest.inventorycube("jeija_wireless_inverter_on.png"),
	material = minetest.digprop_constanttime(0.8),
})

minetest.register_craft({
	output = 'node "jeija:wireless_inverter_off" 2',
	recipe = {
		{'', 'craft "default:steel_ingot"', ''},
		{'craft "jeija:ic"', 'node "jeija:mesecon_off"', 'craft "jeija:ic"'},
		{'', 'node "jeija:mesecon_off"', ''},
	}
})

minetest.register_on_placenode(function(pos, newnode, placer)
	if newnode.name == "jeija:wireless_inverter_on" then
		mesecon:receptor_on(pos)
		mesecon:register_wireless_receiver(pos, 1)
	end
end)

minetest.register_on_dignode(
	function(pos, oldnode, digger)
		if oldnode.name == "jeija:wireless_inverter_on" then
			mesecon:remove_wireless_receiver(pos)
			mesecon:receptor_off(pos)
		end	
		if oldnode.name == "jeija:wireless_inverter_off" then
			mesecon:remove_wireless_receiver(pos)		
		end
	end
)

mesecon:add_receptor_node("jeija:wireless_inverter_on")

-- WIRELESS TRANSMITTER

function mesecon:wireless_transmit(channel, senderstate)
	local i = 1
	while mesecon.wireless_receivers[i]~=nil do
		if mesecon.wireless_receivers[i].channel==channel then
			if senderstate==1 then
				mesecon.wireless_receivers[i].requested_state=1
			elseif senderstate==0 then
				mesecon.wireless_receivers[i].requested_state=0
			end
			nodeupdate(mesecon.wireless_receivers[i].pos)
		end
		i=i+1
	end
end

minetest.register_node("jeija:wireless_transmitter_on", {
	tile_images = {"jeija_wireless_transmitter_tb.png", "jeija_wireless_transmitter_tb.png", "jeija_wireless_transmitter_on.png", "jeija_wireless_transmitter_on.png", "jeija_wireless_transmitter_on.png", "jeija_wireless_transmitter_on.png"},
	inventory_image = minetest.inventorycube("jeija_wireless_transmitter_on.png"),
	material = minetest.digprop_constanttime(0.8),
})

minetest.register_node("jeija:wireless_transmitter_off", {
	tile_images = {"jeija_wireless_transmitter_tb.png", "jeija_wireless_transmitter_tb.png", "jeija_wireless_transmitter_off.png", "jeija_wireless_transmitter_off.png", "jeija_wireless_transmitter_off.png", "jeija_wireless_transmitter_off.png"},
	inventory_image = minetest.inventorycube("jeija_wireless_transmitter_off.png"),
	material = minetest.digprop_constanttime(0.8),
})

minetest.register_craft({
	output = 'node "jeija:wireless_transmitter_off" 2',
	recipe = {
		{'craft "default:steel_ingot"', 'node "jeija:mesecon_off"', 'craft "default:steel_ingot"'},
		{'', 'node "jeija:mesecon_off"', ''},
		{'', 'craft "jeija:ic"', ''},
	}
})

mesecon:register_on_signal_on(function(pos, node)
	if node.name=="jeija:wireless_transmitter_off" then
		minetest.env:add_node(pos, {name="jeija:wireless_transmitter_on"})
		local node_under_pos=pos
		node_under_pos.y=node_under_pos.y-1
		local node_under=minetest.env:get_node(node_under_pos)
		mesecon:wireless_transmit(node_under.name, 1)
	end
end)

mesecon:register_on_signal_off(function(pos, node)
	if node.name=="jeija:wireless_transmitter_on" then
		minetest.env:add_node(pos, {name="jeija:wireless_transmitter_off"})
		local node_under_pos=pos
		node_under_pos.y=node_under_pos.y-1
		local node_under=minetest.env:get_node(node_under_pos)
		mesecon:wireless_transmit(node_under.name, 0)
	end
end)



--INIT
mesecon:read_wlre_from_file()
minetest.register_on_newplayer(function(player)
	local i=1
	while mesecon.wireless_receivers[i]~=nil do
		pos=mesecon.wireless_receivers[i].pos
		request=mesecon.wireless_receivers[i].requested_state
		inverting=mesecon.wireless_receivers[i].inverting
		if request==inverting then
		mesecon:receptor_off(pos)
		end
		if request~=inverting  then
			mesecon:receptor_off(pos)
		end
	end
end)
